using System;
using QTPlugin.Interop;

namespace QTPlugin
{
	/// <summary>
	/// Represents the method that will handle events for IPluginServer.
	/// </summary>
	/// <param name="sender"></param>
	/// <param name="e"></param>
	public delegate void PluginEventHandler( object sender, PluginEventArgs e );

	/// <summary>
	/// Provides data for IPluginServer events. 
	/// </summary>
	public sealed class PluginEventArgs : EventArgs
	{
		private int index;
		private Address address;
		private ExplorerWindowActions windowAction;

		/// <summary>
		/// 
		/// </summary>
		/// <param name="index"></param>
		/// <param name="address"></param>
		public PluginEventArgs( int index, Address address )
		{
			this.index = index;
			this.address = address;
		}

		/// <summary>
		/// 
		/// </summary>
		/// <param name="windowAction"></param>
		public PluginEventArgs( ExplorerWindowActions windowAction )
		{
			this.windowAction = windowAction;
		}

		/// <summary>
		/// Tab index.
		/// 
		/// On SettingsChanged, 0 is passed when TabBar Option changed and 1 when ButtonBar Option changed.
		/// </summary>
		public int Index
		{
			get
			{
				return this.index;
			}
		}

		/// <summary>
		/// Tab address.
		/// </summary>
		public Address Address
		{
			get
			{
				return this.address;
			}
		}

		/// <summary>
		/// Explorer window actions. only used by ExplorerStateChanged event.
		/// </summary>
		public ExplorerWindowActions WindowAction
		{
			get
			{
				return this.windowAction;
			}
		}
	}

}