using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Windows.Forms;
using Microsoft.Win32;

using QTPlugin;
using System.IO;

namespace QuizoPlugins
{
	partial class FormCreateItems : Form
	{
		private int iStart, iEnd;
		private bool fNowUpdatingNUD;
		private bool fNowUpdating = true;
		private string pathCurrent;

		private static string PLCACEHOLDER = "%n%";
		private static int COUNT_CONFIRM_MIN = 64;

		private IPluginServer pluginServer;


		public FormCreateItems( string pathCurrent, IPluginServer pluginServer )
		{
			this.pluginServer = pluginServer;

			this.pathCurrent = pathCurrent;
			if( !this.pathCurrent.EndsWith( @"\" ) )
				this.pathCurrent += @"\";

			this.Text = this.pathCurrent;

			InitializeComponent();

			if( System.Globalization.CultureInfo.CurrentCulture.Parent.Name == "ja" )
			{
				string[] dialogStrs = Resource.dialog_ja.Split( new char[] { ';' } );

				this.labelBaseName.Text			= dialogStrs[0];
				this.labelStartVal.Text			= dialogStrs[1];
				this.labelEnd.Text				= dialogStrs[2];
				this.labelExt.Text				= dialogStrs[3];
				this.checkBoxAddZero.Text		= dialogStrs[4];
				this.checkBoxConfirm.Text		= dialogStrs[5];
				this.buttonSave.Text			= dialogStrs[6];
				this.buttonCancel.Text			= dialogStrs[7];
				this.textBoxBaseName.Text		= dialogStrs[8];
				this.checkBoxCloseOnCreate.Text = dialogStrs[9];
			}

			this.toolTip1.SetToolTip( this.textBoxBaseName, StringResources.Tooltips[0] );
			this.toolTip1.SetToolTip( this.buttonCreateFolder, StringResources.Tooltips[1] );
			this.toolTip1.SetToolTip( this.buttonCreateEmptyFile, StringResources.Tooltips[2] );
			
			this.ReadSettings();
			this.fNowUpdating = false;
			this.CreateSampleFileName();
		}


		private void FormCreateItems_FormClosing( object sender, FormClosingEventArgs e )
		{
			using( RegistryKey rkPlugin = Registry.CurrentUser.CreateSubKey( CONSTANTS.REGISTRY_PLUGINSETTINGS + "\\" + CreateNewItemButton.REGNAME ) )
			{
				if( rkPlugin != null )
				{
					rkPlugin.SetValue( "x", this.Location.X );
					rkPlugin.SetValue( "y", this.Location.Y );
				}
			}
		}
		
		private void textBoxes_TextChanged( object sender, EventArgs e )
		{
			this.CreateSampleFileName();
		}

		private void numericUpDowns_ValueChanged( object sender, EventArgs e )
		{
			if( this.fNowUpdatingNUD )
				return;

			this.fNowUpdatingNUD = true;

			this.iStart = (int)this.numericUpDownStart.Value;
			this.iEnd = (int)this.numericUpDownEnd.Value;

			if( this.iEnd < this.iStart )
			{
				this.numericUpDownEnd.Value = this.iEnd = this.iStart;
			}
			else
			{
				this.numericUpDownStart.Value = (int)this.iStart;
				this.numericUpDownEnd.Value = (int)this.iEnd;
			}

			this.fNowUpdatingNUD = false;
			this.CreateSampleFileName();
		}

		private void checkBoxAddZero_CheckedChanged( object sender, EventArgs e )
		{
			this.CreateSampleFileName();
		}

		private void buttonCreateFolder_Click( object sender, EventArgs e )
		{
			if( this.iEnd - this.iStart + 1 < COUNT_CONFIRM_MIN || !this.checkBoxConfirm.Checked || ConfirmTooManyFile( this.iEnd - this.iStart + 1 ) )
			{
				this.CreateFiles( false );
			}
		}

		private void buttonCreateEmptyFile_Click( object sender, EventArgs e )
		{
			if( this.iEnd - this.iStart + 1 < COUNT_CONFIRM_MIN || !this.checkBoxConfirm.Checked || ConfirmTooManyFile( this.iEnd - this.iStart + 1 ) )
			{
				this.CreateFiles( true );
			}
		}

		private void buttonSave_Click( object sender, EventArgs e )
		{
			using( RegistryKey rkPlugin = Registry.CurrentUser.CreateSubKey( CONSTANTS.REGISTRY_PLUGINSETTINGS + "\\" + CreateNewItemButton.REGNAME ) )
			{
				if( rkPlugin != null )
				{
					rkPlugin.SetValue( "AddZero", this.checkBoxAddZero.Checked ? 1 : 0 );
					rkPlugin.SetValue( "BaseName", this.textBoxBaseName.Text );
					rkPlugin.SetValue( "Confirm", this.checkBoxConfirm.Checked ? 1 : 0 );
					rkPlugin.SetValue( "CloseOnCreate", this.checkBoxCloseOnCreate.Checked ? 1 : 0 );
					rkPlugin.SetValue( "Ext", this.textBoxExt.Text );
					rkPlugin.SetValue( "iStart", this.iStart );
					rkPlugin.SetValue( "iEnd", this.iEnd );
				}
			}
		}

		private void buttonCancel_Click( object sender, EventArgs e )
		{
			this.Close();
		}

		private void textBoxes_KeyPress( object sender, KeyPressEventArgs e )
		{
			if( !CreateNewItemButton.IsValidFileNameChar( e.KeyChar ) )//&& e.KeyChar != (char)Keys.Back )
			{
				e.Handled = true;
				System.Media.SystemSounds.Hand.Play();
			}
		}




		private void ReadSettings()
		{
			using( RegistryKey rkPlugin = Registry.CurrentUser.OpenSubKey( CONSTANTS.REGISTRY_PLUGINSETTINGS + "\\" + CreateNewItemButton.REGNAME ) )
			{
				if( rkPlugin != null )
				{
					int iAddzero = (int)rkPlugin.GetValue( "AddZero", 1 );
					int iConfirm = (int)rkPlugin.GetValue( "Confirm", 1 );
					int iClose = (int)rkPlugin.GetValue( "CloseOnCreate", 0 );
					string strBaseName = (string)rkPlugin.GetValue( "BaseName" );
					string strExt = (string)rkPlugin.GetValue( "Ext" );
					int x = (int)rkPlugin.GetValue( "x", -1 );
					int y = (int)rkPlugin.GetValue( "y", -1 );


					if( !String.IsNullOrEmpty( strBaseName ) )
						this.textBoxBaseName.Text = strBaseName;
					if( !String.IsNullOrEmpty( strExt ) )
						this.textBoxExt.Text = strExt;

					this.iStart = (int)rkPlugin.GetValue( "iStart", 0 );
					this.iEnd = (int)rkPlugin.GetValue( "iEnd", 0 );

					this.fNowUpdatingNUD = true;
					this.numericUpDownEnd.Value = this.iEnd;
					this.numericUpDownStart.Value = this.iStart;
					this.fNowUpdatingNUD = false;

					this.checkBoxAddZero.Checked = iAddzero == 1;
					this.checkBoxConfirm.Checked = iConfirm == 1;
					this.checkBoxCloseOnCreate.Checked = iClose == 1;

					if( x != -1 && y != -1 )
					{
						this.Location = new Point( x, y );
					}
					else
						this.StartPosition = FormStartPosition.CenterScreen;
				}
				else
					this.StartPosition = FormStartPosition.CenterScreen;
			}
		}

		private void CreateFiles( bool fFile )
		{
			if( Directory.Exists( this.pathCurrent ) )
			{
				List<string> lstExisted = new List<string>();
				List<string> lstError = new List<string>();
				List<Address> lstSuccess = new List<Address>();

				string strBaseName = this.MakeFileNameFormat();

				for( int i = this.iStart; i < this.iEnd + 1; i++ )
				{
					string strFileName = String.Format( strBaseName, i );
					if( fFile )
						strFileName += this.textBoxExt.Text;

					string strNewPath = this.pathCurrent + strFileName;

					try
					{
						if( ( fFile && File.Exists( strNewPath ) ) || ( !fFile && Directory.Exists( strNewPath ) ) )
						{
							lstExisted.Add( strFileName );
							continue;
						}

						if( fFile )
						{
							using( File.Create( strNewPath ) )
							{
							}
						}
						else
						{
							Directory.CreateDirectory( strNewPath );
						}
					}
					catch
					{
						lstError.Add( strNewPath );
						continue;
					}

					lstSuccess.Add( new Address( strNewPath ) );
				}

				if( lstSuccess.Count > 0 )
				{
					this.pluginServer.ExecuteCommand( Commands.RefreshBrowser, null );
					this.pluginServer.TrySetSelection( lstSuccess.ToArray(), true );
				}

				if( lstExisted.Count > 0 )
				{
					string strMsg = String.Empty;
					foreach( string s in lstExisted )
					{
						strMsg += s + "\r\n";
					}

					MessageBox.Show( StringResources.Messages[0] + strMsg, String.Empty, MessageBoxButtons.OK, MessageBoxIcon.Information );
				}
				if( lstError.Count > 0 )
				{
					string strMsg = String.Empty;
					foreach( string s in lstError )
					{
						strMsg += s + "\r\n";
					}

					MessageBox.Show( StringResources.Messages[1] + strMsg, String.Empty, MessageBoxButtons.OK, MessageBoxIcon.Error );
				}

				if( lstSuccess.Count > 0 && this.checkBoxCloseOnCreate.Checked )
				{
					this.Close();
				}
			}
			else
			{
				MessageBox.Show( StringResources.Messages[3], String.Empty, MessageBoxButtons.OK, MessageBoxIcon.Error );
			}
		}

		private string MakeFileNameFormat()
		{
			string strBaseName = this.textBoxBaseName.Text;
			strBaseName = strBaseName.Replace( "{", "{{" );
			strBaseName = strBaseName.Replace( "}", "}}" );

			string strFormat = String.Empty;

			if( this.checkBoxAddZero.Checked )
			{
				for( int i = 0; i < this.iEnd.ToString().Length; i++ )
				{
					strFormat += "0";
				}
				strFormat = "{0:" + strFormat + "}";
			}
			else
			{
				strFormat = "{0}";
			}

			if( strBaseName.Contains( PLCACEHOLDER ) )
			{
				strBaseName = strBaseName.Replace( PLCACEHOLDER, strFormat );
			}
			else
			{
				strBaseName += strFormat;
			}

			return strBaseName;
		}

		private static bool ConfirmTooManyFile( int c )
		{
			return DialogResult.OK == MessageBox.Show( String.Format( StringResources.Messages[2], c ), "", MessageBoxButtons.OKCancel );
		}

		private void CreateSampleFileName()
		{
			if( this.fNowUpdating )
				return;

			if( this.textBoxBaseName.Text.Length > 0 )
			{
				string strBaseName = this.MakeFileNameFormat();
				this.textBoxInfo.Text = String.Format( strBaseName, this.iStart ) + "\r\n  ~\r\n" + String.Format( strBaseName, this.iEnd ) + "\r\n\r\n" + ( this.iEnd - this.iStart + 1 ) + " items.";

			}
			else
			{
				this.textBoxInfo.Clear();
			}
		}

	}
}