using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.IO;
using System.Text.RegularExpressions;
using System.Windows.Forms;
using QTPlugin;

namespace QuizoPlugins
{
	sealed class AppLauncher
	{
		private static Regex reFile = new Regex( "%f%", RegexOptions.IgnoreCase );
		private static Regex reDirs = new Regex( "%d%", RegexOptions.IgnoreCase );
		private static Regex reSlct = new Regex( "%s%", RegexOptions.IgnoreCase );
		private static Regex reCrnt = new Regex( "%c%", RegexOptions.IgnoreCase );
		private static Regex reDrCr = new Regex( "%cd%", RegexOptions.IgnoreCase );
		private static Regex reAll = new Regex( "%(f|d|s|c|cd)%", RegexOptions.IgnoreCase );

		private string strCurrentPath;
		private string strSelFiles;
		private string strSelDirs;
		private string strSelObjs;
		private int iSelItemsCount;
		private int iSelFileCount;
		private int iSelDirsCount;


		private AppLauncher( Address[] addresses, string pathCurrent )
		{
			List<string> lstTmpSelected_File = new List<string>();
			List<string> lstTmpSelected_Dirs = new List<string>();

			this.strCurrentPath =
			this.strSelFiles =
			this.strSelDirs =
			this.strSelObjs = String.Empty;

			this.iSelItemsCount =
			this.iSelFileCount =
			this.iSelDirsCount = 0;

			if( addresses != null )
			{
				this.iSelItemsCount = addresses.Length;
				for( int i = 0; i < addresses.Length; i++ )
				{
					if( File.Exists( addresses[i].Path ) )
						lstTmpSelected_File.Add( addresses[i].Path );
					else if( Directory.Exists( addresses[i].Path ) )
						lstTmpSelected_Dirs.Add( addresses[i].Path );
				}
			}

			if( Directory.Exists( pathCurrent ) )
				this.strCurrentPath = "\"" + pathCurrent + "\"";

			foreach( string path in lstTmpSelected_File )
			{
				this.strSelFiles += "\"" + path + "\" ";
			}
			foreach( string path in lstTmpSelected_Dirs )
			{
				this.strSelDirs += "\"" + path + "\" ";
			}
			this.strSelObjs = ( this.strSelFiles + this.strSelDirs ).Trim();
			this.strSelFiles = this.strSelFiles.Trim();
			this.strSelDirs = this.strSelDirs.Trim();

			this.iSelFileCount = lstTmpSelected_File.Count;
			this.iSelDirsCount = lstTmpSelected_Dirs.Count;
		}

		/// <summary>
		/// Replace tokens in workingdirectory with paths.
		/// </summary>
		/// <param name="mia"></param>
		/// <returns>true if some tokens are replaced.</returns>
		private void ReplaceTokens_WorkingDir( ProcessStartInfo psi )
		{
			if( !String.IsNullOrEmpty( psi.WorkingDirectory ) )
			{
				bool fReplaced = false;
				if( iSelItemsCount == 1 )
				{
					if( reSlct.IsMatch( psi.WorkingDirectory ) && Directory.Exists( strSelDirs.Trim( new char[] { '"' } ) ) )
					{
						// %s%
						psi.WorkingDirectory = reSlct.Replace( psi.WorkingDirectory, strSelDirs );
						fReplaced = true;
					}
					else if( reDirs.IsMatch( psi.WorkingDirectory ) && strSelDirs.Length > 0 )
					{
						// %d%
						psi.WorkingDirectory = reDirs.Replace( psi.WorkingDirectory, strSelDirs );
						fReplaced = true;
					}
					else if( reDrCr.IsMatch( psi.WorkingDirectory ) && Directory.Exists( strSelDirs.Trim( new char[] { '"' } ) ) )
					{
						// %cd%, sel dir
						psi.WorkingDirectory = reDrCr.Replace( psi.WorkingDirectory, strSelDirs );
						fReplaced = true;
					}
				}

				if( !fReplaced && strCurrentPath != null )
				{
					if( reDrCr.IsMatch( psi.WorkingDirectory ) )
					{
						// %cd%, current dir
						psi.WorkingDirectory = reDrCr.Replace( psi.WorkingDirectory, strCurrentPath );
					}
					else if( reCrnt.IsMatch( psi.WorkingDirectory ) )
					{
						// %c%
						psi.WorkingDirectory = reCrnt.Replace( psi.WorkingDirectory, strCurrentPath );
					}
				}

				psi.WorkingDirectory = reAll.Replace( psi.WorkingDirectory, String.Empty );
			}
		}

		/// <summary>
		/// Replace tokens in arguments with paths.
		/// </summary>
		/// <param name="mia"></param>
		/// <returns></returns>
		private void ReplaceTokens_Arguments( ProcessStartInfo psi )
		{
			if( !String.IsNullOrEmpty( psi.Arguments ) )
			{
				if( iSelItemsCount > 0 )
				{
					// %s%
					if( reSlct.IsMatch( psi.Arguments ) )
					{
						psi.Arguments = reSlct.Replace( psi.Arguments, strSelObjs );
					}

					// %f%
					if( reFile.IsMatch( psi.Arguments ) )
					{
						psi.Arguments = reFile.Replace( psi.Arguments, strSelFiles );
					}

					// %d%
					if( reDirs.IsMatch( psi.Arguments ) )
					{
						psi.Arguments = reDirs.Replace( psi.Arguments, strSelDirs );
					}

					// %cd%
					if( reDrCr.IsMatch( psi.Arguments ) )
					{
						if( strSelDirs.Length > 0 )
						{
							psi.Arguments = reDrCr.Replace( psi.Arguments, strSelDirs );
						}
						else if( strCurrentPath != null )
						{
							psi.Arguments = reDrCr.Replace( psi.Arguments, strCurrentPath );
						}
					}
				}
				else
				{
					// %cd%
					if( strCurrentPath != null && reDrCr.IsMatch( psi.Arguments ) )
					{
						psi.Arguments = reDrCr.Replace( psi.Arguments, strCurrentPath );
					}
				}

				// %c%
				if( strCurrentPath != null && reCrnt.IsMatch( psi.Arguments ) )
				{
					psi.Arguments = reCrnt.Replace( psi.Arguments, strCurrentPath );
				}

				psi.Arguments = reAll.Replace( psi.Arguments, String.Empty );
			}
		}

		/// <summary>
		/// 
		/// </summary>
		/// <param name="pluginServer"></param>
		/// <param name="psi"></param>
		public static void ReplcaceTokens( IPluginServer pluginServer, ProcessStartInfo psi )
		{
			Address[] addresses;
			string pathCurrent = String.Empty;

			if( !pluginServer.TryGetSelection( out addresses ) )
			{
				addresses = new Address[0];
			}

			ITab tabCurrent = pluginServer.SelectedTab;
			if( tabCurrent != null )
			{
				pathCurrent = tabCurrent.Address.Path;
			}

			AppLauncher al = new AppLauncher( addresses, pathCurrent );
			al.ReplaceTokens_WorkingDir( psi );
			al.ReplaceTokens_Arguments( psi );

			// directory string is quoted by '"'
			if( !String.IsNullOrEmpty( psi.WorkingDirectory ) )
				psi.WorkingDirectory = psi.WorkingDirectory.Trim( new char[] { '"' } );
		}

	}
}