using System;
using System.Collections.Generic;
using System.Drawing;
using System.IO;
using System.Runtime.InteropServices;
using System.Runtime.Serialization.Formatters.Binary;
using System.Text;
using Microsoft.Win32;

namespace QuizoPlugins
{
	/// <summary>
	/// 
	/// </summary>
	static class PInvoke
	{
		[DllImport( "shell32.dll", CharSet = CharSet.Auto )]
		public extern static int ExtractIconEx(
			[MarshalAs( UnmanagedType.LPTStr )] 
            string lpszFile,       //size of the icon
			int nIconIndex,        //index of the icon 
			//(in case we have more 
			//then 1 icon in the file
			IntPtr[] phIconLarge,  //32x32 icon
			IntPtr[] phIconSmall,  //16x16 icon
			int nIcons );           //how many to get

		[DllImport( "user32.dll", CharSet = CharSet.Auto )]
		public extern static bool DestroyIcon( IntPtr hIcon );

		/// <summary>
		/// Maps the specified executable module into the address space of the calling process.
		/// The executable module can be a .dll or an .exe file.
		/// The specified module may cause other modules to be mapped into the address space.
		/// </summary>
		/// <param name="lpFileName"></param>
		/// <param name="hFile">This parameter is reserved for future use. It must be NULL.</param>
		/// <param name="dwFlags"></param>
		/// <returns></returns>
		[DllImport( "kernel32.dll", CharSet = CharSet.Auto )]
		public extern static IntPtr LoadLibraryEx(
			[MarshalAs( UnmanagedType.LPTStr )] 
			string lpFileName,
			IntPtr hFile,
			int dwFlags );

		[DllImport( "kernel32.dll", CharSet = CharSet.Auto )]
		public extern static bool FreeLibrary( IntPtr hModule );

		[DllImport( "kernel32.dll", CharSet = CharSet.Auto )]
		public extern static bool EnumResourceNames(
			IntPtr hModule,
			IntPtr lpszType,
			EnumResNameProc lpEnumFunc,
			IntPtr lParam );

		[DllImport( "kernel32.dll", CharSet = CharSet.Auto )]
		public extern static IntPtr FindResource(
			IntPtr hModule,
			IntPtr lpName,
			IntPtr lpType );

		[DllImport( "kernel32.dll", CharSet = CharSet.Auto )]
		public extern static IntPtr FindResource(
			IntPtr hModule,
			string lpName,
			IntPtr lpType );

		[DllImport( "kernel32.dll", CharSet = CharSet.Auto )]
		public extern static IntPtr LoadResource( IntPtr hModule, IntPtr hResInfo );

		[DllImport( "kernel32.dll", CharSet = CharSet.Auto )]
		public extern static IntPtr LockResource( IntPtr hResData );

		[DllImport( "kernel32.dll", CharSet = CharSet.Auto )]
		public extern static int SizeofResource( IntPtr hModule, IntPtr hResInfo );

		[DllImport( "user32.dll", CharSet = CharSet.Auto )]
		public extern static uint LookupIconIdFromDirectoryEx( IntPtr pResBits, bool fIcon, int cxDesired, int cyDesired, int Flags );

		[DllImport( "user32.dll", CharSet = CharSet.Auto )]
		public extern static IntPtr CreateIconFromResource( IntPtr pResBits, int dwResSize, bool fIcon, int dwVer );

	}

	/// <summary>
	/// An application-defined callback function used with the EnumResourceNames and EnumResourceNamesEx functions. 
	/// It receives the type and name of a resource. 
	/// The ENUMRESNAMEPROC type defines a pointer to this callback function.
	/// 
	/// REMARKS
	/// 
	/// If IS_INTRESOURCE(lpszType) is TRUE, then lpszType specifies the integer identifier of the given resource type.
	/// Otherwise, it is a pointer to a null-terminated string. 
	/// If the first character of the string is a pound sign (#), then the remaining characters represent a decimal number
	/// that specifies the integer identifier of the resource type. For example, the string "#258" represents the identifier 258.
	/// Similarly, if IS_INTRESOURCE(lpszName) is TRUE, then lpszName specifies the integer identifier of the given resource.
	/// Otherwise, it is a pointer to a null-terminated string. If the first character of the string is a pound sign (#),
	/// then the remaining characters represent a decimal number that specifies the integer identifier of the resource.
	/// An application must register this function by passing its address to the EnumResourceNames or EnumResourceNamesEx function.
	/// </summary>
	/// <param name="hModule">
	/// The handle to the module whose executable file contains the resources that are being enumerated. 
	/// If this parameter is NULL, the function enumerates the resource names in the module used to create the current process. 
	/// </param>
	/// <param name="lpszType">
	/// Pointer to a null-terminated string specifying the type of resource that is being enumerated.
	/// For standard resource types, see Resource Types. For more information, see the Remarks section above. 
	/// </param>
	/// <param name="lpszName">
	/// Specifies the name of a resource of the type being enumerated. For more information, see the Remarks section above. 
	/// </param>
	/// <param name="lParam">
	/// Specifies the application-defined parameter passed to the EnumResourceNames or EnumResourceNamesEx function.
	/// This parameter can be used in error checking. 
	/// </param>
	/// <returns>
	/// Returns TRUE to continue enumeration or FALSE to stop enumeration.
	/// </returns>
	delegate bool EnumResNameProc( IntPtr hModule, IntPtr lpszType, IntPtr lpszName, IntPtr lParam );

	/// <summary>
	/// 
	/// </summary>
	static class Util
	{
		/// <summary>
		/// 
		/// </summary>
		/// <typeparam name="T"></typeparam>
		/// <param name="regValueName"></param>
		/// <param name="rkUserApps"></param>
		/// <returns></returns>
		public static T[] ReadRegBinary<T>( string regValueName, RegistryKey rkUserApps )
		{
			byte[] regValue = null;
			try
			{
				regValue = (byte[])rkUserApps.GetValue( regValueName, null );
			}
			catch
			{
				return null;
			}

			if( regValue != null && regValue.Length > 0 )
			{
				using( MemoryStream stream = new MemoryStream( regValue ) )
				{
					BinaryFormatter formatter = new BinaryFormatter();
					return (T[])formatter.Deserialize( stream );
				}
			}
			return null;
		}

		/// <summary>
		/// 
		/// </summary>
		/// <typeparam name="T"></typeparam>
		/// <param name="array"></param>
		/// <param name="regValueName"></param>
		/// <param name="rkUserApps"></param>
		public static void WriteRegBinary<T>( T[] array, string regValueName, RegistryKey rkUserApps )
		{
			if( array != null )
			{
				byte[] bytes;
				BinaryFormatter formatter = new BinaryFormatter();

				using( MemoryStream stream = new MemoryStream() )
				{
					formatter.Serialize( stream, array );
					bytes = stream.GetBuffer();
				}

				int lastIndex = 0;
				for( int i = 0; i < bytes.Length; i++ )
				{
					if( bytes[i] == 0 )
					{
						if( lastIndex == 0 )
							lastIndex = i;
					}
					else
					{
						lastIndex = 0;
					}
				}


				byte[] newBytes = new byte[lastIndex];
				if( lastIndex != 0 )
				{
					for( int i = 0; i < lastIndex; i++ )
					{
						newBytes[i] = bytes[i];
					}
				}
				else
				{
					newBytes = bytes;
				}

				if( rkUserApps != null )
				{
					rkUserApps.SetValue( regValueName, newBytes );
				}
			}
		}

		/// <summary>
		/// 
		/// </summary>
		/// <param name="path"></param>
		/// <returns></returns>
		//public static string SanitizePathString( string path )
		//{
		//    if( path == null )
		//        return null;

		//    path = path.Trim();

		//    StringBuilder sb = new StringBuilder( path.Length );
		//    for( int i = 0; i < path.Length; i++ )
		//    {
		//        if( Util.IsValidPathChar( path[i] ) && path[i] > 0x1f )
		//        {
		//            // ", <, >, |, *, ? 

		//            sb.Append( path[i] );
		//        }
		//    }
		//    return sb.ToString();
		//}

		/// <summary>
		/// 
		/// </summary>
		/// <param name="ch"></param>
		/// <returns></returns>
		//public static bool IsValidPathChar( char ch )
		//{
		//    // ", <, >, |, *, ? 

		//    // allowed: \, /, : 

		//    return ( ch != '"' ) && ( ch != '<' ) && ( ch != '>' ) && ( ch != '|' ) && ( ch != '*' ) && ( ch != '?' );// && ( ch > 0x1f );
		//}

		public static Icon GetIcon( string path, bool fLarge )
		{
			if( String.IsNullOrEmpty( path ) )
				return null;

			path = Environment.ExpandEnvironmentVariables( path );

			SHFILEINFO shinfo = new SHFILEINFO();
			Icon returnIcon;

			const int SHGFI_ICON = 0x000000100;
			const int SHGFI_SMALLICON = 0x000000001;
			const int SHGFI_PIDL = 0x000000008;	// pszPath is a pidl

			const string INDICATOR_NAMESPACE = "::";


			int flag = fLarge ? SHGFI_ICON : SHGFI_ICON | SHGFI_SMALLICON;

			if( path.StartsWith( INDICATOR_NAMESPACE ) )
			{
				IntPtr pIDL = Util.ILCreateFromPath( path );
				if( pIDL != IntPtr.Zero )
				{
					if( IntPtr.Zero != Util.SHGetFileInfo( pIDL, 0, ref shinfo, Marshal.SizeOf( shinfo ), flag | SHGFI_PIDL ) &&
						shinfo.hIcon != IntPtr.Zero )
					{
						returnIcon = new Icon( Icon.FromHandle( shinfo.hIcon ), 16, 16 );
						Util.DestroyIcon( shinfo.hIcon );
						Marshal.FreeCoTaskMem( pIDL );

						return returnIcon;
					}
					Marshal.FreeCoTaskMem( pIDL );
				}
			}
			else if( IntPtr.Zero != Util.SHGetFileInfo( path, 0, ref shinfo, Marshal.SizeOf( shinfo ), flag ) && shinfo.hIcon != IntPtr.Zero )
			{
				returnIcon = new Icon( Icon.FromHandle( shinfo.hIcon ), 16, 16 );
				Util.DestroyIcon( shinfo.hIcon );
				return returnIcon;
			}

			return null;
		}

		[DllImport( "shell32.dll", CharSet = CharSet.Unicode )]
		private static extern IntPtr SHGetFileInfo( string pszPath, uint dwFileAttributes, ref SHFILEINFO psfi, int cbSizeFileInfo, int uFlags );

		[DllImport( "shell32.dll", CharSet = CharSet.Unicode )]
		private static extern IntPtr SHGetFileInfo( IntPtr pszPath, uint dwFileAttributes, ref SHFILEINFO psfi, int cbSizeFileInfo, int uFlags );

		[DllImport( "shell32.dll", CharSet = CharSet.Unicode )]
		private static extern IntPtr ILCreateFromPath( string pszPath );

		[DllImport( "user32.dll" )]
		private static extern int DestroyIcon( IntPtr hIcon );

		//[DllImport( "user32.dll" )]
		//public static extern bool GetWindowRect( IntPtr hWnd, out RECT pRect );

	}

	[StructLayout( LayoutKind.Sequential, CharSet = CharSet.Unicode )]		//Pack = 1 on Win32
	struct SHFILEINFO
	{
		public IntPtr hIcon;
		public int iIcon;
		public uint dwAttributes;
		[MarshalAs( UnmanagedType.ByValTStr, SizeConst = 260 )]
		public string szDisplayName;
		[MarshalAs( UnmanagedType.ByValTStr, SizeConst = 80 )]
		public string szTypeName;
	}


}