using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Windows.Forms;
using System.IO;
using System.Runtime.InteropServices;

namespace QuizoPlugins
{
	partial class SelectIconDialog
	{
		partial class SelectIconDialogInnerForm : Form
		{
			private string strPath;
			private string strPathRet;
			private int iIconIndex = -1;
			private bool fLargeIcon;

			private static Localizer localizer = new Localizer();
			

			public SelectIconDialogInnerForm( string path, int iconIndexToSelect, bool fLargeIcon )
			{
				this.strPath = path;
				this.fLargeIcon = fLargeIcon;
				this.iIconIndex = iconIndexToSelect;

				InitializeComponent();


				this.StartPosition = FormStartPosition.CenterParent;
			}


			private void ExtractIcons( string path )
			{
				List<Image> lstImages = new List<Image>();
				
				// get icons from executables.
				string ext = Path.GetExtension( path ).ToLower();
				if( ext == ".dll" || ext == ".exe" )
				{
					this.ExtractIcons_DLLEXE( path );
					return;
				}
				else if( EXTS_IMAGE.Contains( ext ) )
				{
					try
					{
						using( Bitmap bmpRaw = new Bitmap( path ) )
						{
							Size size = bmpRaw.Size;
							if( size == new Size( 16, 16 ) || size == new Size( 24, 24 ) || size == new Size( 32, 32 ) || size == new Size( 48, 48 ) )
							{
								Bitmap bmp = new Bitmap( bmpRaw );
								if( ext == ".bmp" )
									bmp.MakeTransparent( Color.Magenta );

								lstImages.Add( bmp );
							}
						}
					}
					catch
					{
					}
					this.CreateViewItems( lstImages );
					return;
				}

				int iIconCount = PInvoke.ExtractIconEx( path, -1, null, null, 0 );
				if( iIconCount > 0 )
				{
					IntPtr[] ppIcons = new IntPtr[iIconCount];

					int iExtracted = 0;
					if( this.fLargeIcon )
						iExtracted = PInvoke.ExtractIconEx( path, 0, ppIcons, null, iIconCount );
					else
						iExtracted = PInvoke.ExtractIconEx( path, 0, null, ppIcons, iIconCount );

					for( int i = 0; i < iExtracted; i++ )
					{
						if( ppIcons[i] != IntPtr.Zero )
						{
							using( Icon icoTemp = Icon.FromHandle( ppIcons[i] ) )
							{
								if( icoTemp != null )
								{
									lstImages.Add( icoTemp.ToBitmap() );
								}
							}
							// Disposing icon constructed using Icon.FromHandle()
							// does not call DestroyIcon API.
							PInvoke.DestroyIcon( ppIcons[i] );
						}
					}
				}
				this.CreateViewItems( lstImages );
			}

			private void ExtractIcons_DLLEXE( string path )
			{
				List<IconPair> lstHICON = SelectIconDialog.GetIconHandles( path, -1, false );
				List<Image> lstImages = new List<Image>();

				if( lstHICON != null )
				{
					for( int i = 0; i < lstHICON.Count; i++ )
					{
						using( lstHICON[i] )
						{
							IntPtr hIcon = this.fLargeIcon ? lstHICON[i].hIconLarge : lstHICON[i].hIconSmall;
							if( hIcon != IntPtr.Zero )
							{
								using( Icon icoTemp = Icon.FromHandle( hIcon ) )
								{
									if( icoTemp != null )
									{
										lstImages.Add( icoTemp.ToBitmap() );
									}
								}
								// Disposing icon constructed using Icon.FromHandle()
								// does not call DestroyIcon API.
							}
						}
					}
				}
				this.CreateViewItems( lstImages );
			}

			private void SetReturnIcon()
			{
				if( this.listViewIcons.SelectedItems.Count == 1 )
				{
					this.iIconIndex = this.listViewIcons.SelectedItems[0].Index;
					this.strPathRet = this.strPath;
					this.DialogResult = DialogResult.OK;
				}
				else
				{
					this.iIconIndex = -1;
					this.strPathRet = String.Empty;
					this.DialogResult = DialogResult.Cancel;
				}
			}

			/// <summary>
			/// 
			/// </summary>
			/// <param name="lstImages">images in this collection are added to ImageList.</param>
			private void CreateViewItems( List<Image> lstImages )
			{
				this.listViewIcons.SuspendLayout();
				this.listViewIcons.BeginUpdate();
				this.listViewIcons.Items.Clear();

				// Clear image lists.
				for( int i = 0; i < this.imageListLarge.Images.Count; i++ )
				{
					this.imageListLarge.Images[i].Dispose();
				}
				this.imageListLarge.Images.Clear();
				for( int i = 0; i < this.imageListSmall.Images.Count; i++ )
				{
					this.imageListSmall.Images[i].Dispose();
				}
				this.imageListSmall.Images.Clear();

				if( this.fLargeIcon )
				{
					this.imageListLarge.Images.AddRange( lstImages.ToArray() );
					this.listViewIcons.View = View.LargeIcon;
				}
				else
				{
					this.imageListSmall.Images.AddRange( lstImages.ToArray() );
					this.listViewIcons.View = View.SmallIcon;
				}

				for( int i = 0; i < lstImages.Count; i++ )
				{
					if( lstImages[i] != null )
					{
						ListViewItem lvi = new ListViewItem( i.ToString() );
						lvi.ImageIndex = i;

						this.listViewIcons.Items.Add( lvi );
					}
				}

				this.listViewIcons.EndUpdate();
				this.listViewIcons.ResumeLayout();
			}


			#region --------- Event Handlers ---------

			private void SelectIconDialogInnerForm_Load( object sender, EventArgs e )
			{
				string[] resStrs = localizer.DialogResources;
				if( resStrs != null && resStrs.Length == 6 )
				{
					this.Text = resStrs[0];
					this.lblPath.Text = resStrs[1];
					this.lblIcon.Text = resStrs[2];
					this.btnBrowse.Text = resStrs[3];
					this.btnOK.Text = resStrs[4];
					this.btnCancel.Text = resStrs[5];
				}

				if( !String.IsNullOrEmpty( this.strPath ) )
				{
					this.tbPath.Text = this.strPath;

					string path = Environment.ExpandEnvironmentVariables( this.strPath );
					if( File.Exists( path ) )
					{
						this.ExtractIcons( path );

						if( -1 < this.iIconIndex && this.iIconIndex < this.listViewIcons.Items.Count )
						{
							this.listViewIcons.Items[this.iIconIndex].Selected = true;
							this.listViewIcons.EnsureVisible( this.iIconIndex );
						}
					}
				}
			}

			private void SelectIconDialogInnerForm_KeyPress( object sender, KeyPressEventArgs e )
			{
				if( e.KeyChar == (char)Keys.Enter )
				{
					e.Handled = true;

					if( this.ActiveControl == this.tbPath )
					{
						if( this.tbPath.Text.Length > 0 )
						{
							string path = Environment.ExpandEnvironmentVariables( this.tbPath.Text );
							if( File.Exists( path ) )
							{
								this.strPath = this.tbPath.Text;
								this.ExtractIcons( path );
								this.listViewIcons.Focus();
							}
						}
					}
					else
					{
						this.SetReturnIcon();
					}
				}
			}

			private void listViewIcons_ItemActivate( object sender, EventArgs e )
			{
				this.SetReturnIcon();
			}

			private void btnBrowse_Click( object sender, EventArgs e )
			{
				using( OpenFileDialog ofd = new OpenFileDialog() )
				{
					if( this.tbPath.Text.Length > 0 )
					{
						if( File.Exists( this.tbPath.Text ) )
						{
							ofd.FileName = this.tbPath.Text;
						}
					}

					ofd.Multiselect = false;
					ofd.RestoreDirectory = false;
					ofd.Filter = "Icon or image files (*.ico,*.exe,*.dll,*.png,*.bmp,*.jpg,*.gif)|*.ico;*.exe;*.dll;*.png;*.bmp;*.jpg;*.gif";

					if( DialogResult.OK == ofd.ShowDialog() )
					{
						this.strPath = this.tbPath.Text = ofd.FileName;
						this.ExtractIcons( this.strPath );
						this.listViewIcons.Focus();
					}
				}
			}

			private void btnOK_Click( object sender, EventArgs e )
			{
				this.SetReturnIcon();
			}

			private void btnCancel_Click( object sender, EventArgs e )
			{
				this.strPath = this.strPathRet = String.Empty;
				this.iIconIndex = -1;
				this.DialogResult = DialogResult.Cancel;
			}

			#endregion


			#region --------- Properties ---------

			public bool LargeIcon
			{
				get
				{
					return this.fLargeIcon;
				}
				set
				{
					this.fLargeIcon = value;
				}
			}

			public string ResourceString
			{
				get
				{
					if( !String.IsNullOrEmpty( this.strPathRet ) )
					{
						if( this.iIconIndex > -1 )
							return this.strPathRet + ", " + this.iIconIndex;
						else
							return this.strPathRet;
					}

					return String.Empty;
				}
			}

			/// <summary>
			/// may be null.
			/// </summary>
			public string FileName
			{
				get
				{
					if( !String.IsNullOrEmpty( this.strPath ) )
						return this.strPath;

					return String.Empty;
				}
				set
				{
					this.strPath = value;
				}
			}

			public int IconIndex
			{
				get
				{
					return this.iIconIndex;
				}
			}

			#endregion

		}
	}
}