using System;
using System.Collections.Generic;
using System.Drawing;
using System.Globalization;
using System.Text;

using QTPlugin;
using QTPlugin.Interop;

namespace QuizoPlugins
{
	[Plugin( PluginType.Interactive, typeof( Localizer ), Version = "1.1.0.0" )]
	public class OptionButton : IBarButton
	{
		private IPluginServer pluginServer;

		#region IPluginClient members

		public void Open( IPluginServer pluginServer, IShellBrowser shellBrowser )
		{
			this.pluginServer = pluginServer;
		}

		public bool QueryShortcutKeys( out string[] actions )
		{
			actions = null;
			return false;
		}

		public void Close( EndCode endCode )
		{
		}

		public bool HasOption
		{
			get
			{
				return false;
			}
		}

		public void OnMenuItemClick( MenuType menuType, string menuText, ITab tab )
		{
		}

		public void OnOption()
		{
		}

		public void OnShortcutKeyPressed( int index )
		{
		}

		#endregion


		#region IBarButton members

		public Image GetImage( bool fLarge )
		{
			return fLarge ? Resource.OptionButton_large : Resource.OptionButton_small;
		}

		public void InitializeItem()
		{
		}

		public void OnButtonClick()
		{
			this.pluginServer.ExecuteCommand( Commands.OpenTabBarOptionDialog, null );
		}

		public bool ShowTextLabel
		{
			get
			{
				return false;
			}
		}

		public string Text
		{
			get
			{
				if( CultureInfo.CurrentCulture.Parent.Name == "ja" )
					return "IvV";
				else
					return "Option";
			}
		}

		#endregion

	}

	sealed class Localizer : LocalizedStringProvider
	{
		private string parent;

		public Localizer()
		{
			this.parent = CultureInfo.CurrentCulture.Parent.Name;
		}

		public override string Author
		{
			get
			{
				return "Quizo";
			}
		}

		public override string Description
		{
			get
			{
				if( this.parent == "ja" )
				{
					return "QT TabBar ̃IvVEBhE\{^ǉ܂B";
				}
				else
				{
					return "Button to show option dialog of QT TabBar.";
				}
			}
		}

		public override string Name
		{
			get
			{
				if( this.parent == "ja" )
				{
					return "IvV {^";
				}
				else
				{
					return "Options Button";
				}
			}
		}

		public override void SetKey( int iKey )
		{			
		}
	}
}
