using System;
using System.Collections.Generic;
using System.Globalization;
using System.Text;
using System.Runtime.InteropServices;

using QTPlugin;
using QTPlugin.Interop;

namespace QuizoPlugins
{
	[Plugin( PluginType.Background, typeof( Localizer ), Version = "1.1.0.0" )]
	public sealed class ShowStatusBar : IPluginClient
	{
		private IPluginServer pluginServer;
		private IShellBrowser shellBrowser;

		private Localizer localizer = new Localizer();


		[DllImport( "user32.dll", CharSet = CharSet.Auto )]
		private static extern IntPtr SendMessage( IntPtr hWnd, UInt32 Msg, IntPtr wParam, IntPtr lParam );

		[DllImport( "user32.dll", CharSet = CharSet.Auto )]
		private static extern bool PostMessage( IntPtr hWnd, UInt32 Msg, IntPtr wParam, IntPtr lParam );

		[DllImport( "user32.dll", CharSet = CharSet.Auto )]
		private static extern IntPtr FindWindowEx( IntPtr hwndParent, IntPtr hwndChildAfter, StringBuilder lpszClass, string lpszWindow );

		private static bool fVista = IsVista();
		private const int WM_COMMAND = 0x0111;

		private static bool IsVista()
		{
			return Environment.OSVersion.Version.Major > 5;
		}


		#region IPluginClient members

		public void Open( IPluginServer pluginServer, IShellBrowser shellBrowser )
		{
			this.pluginServer = pluginServer;
			this.shellBrowser = shellBrowser;

			this.pluginServer.RegisterMenu( this, MenuType.Bar, this.localizer.ShortcutKeyString, true );
		}

		public bool QueryShortcutKeys( out string[] actions )
		{
			actions = new string[] { this.localizer.ShortcutKeyString };
			return true;
		}

		public void Close( EndCode endCode )
		{

		}

		public bool HasOption
		{
			get
			{
				return false;
			}
		}

		public void OnMenuItemClick( MenuType menuType, string menuText, ITab tab )
		{
			this.OnShortcutKeyPressed( 0 );
		}

		public void OnOption()
		{
		}

		public void OnShortcutKeyPressed( int index )
		{
			IntPtr hwndExplr = this.pluginServer.ExplorerHandle;
			IntPtr hwnd = fVista ? FindWindowEx( hwndExplr, IntPtr.Zero, new StringBuilder( "ShellTabWindowClass" ), null ) : hwndExplr;

			int command = 0xA202;

			if( fVista )
				SendMessage( hwnd, WM_COMMAND, (IntPtr)command, IntPtr.Zero );
			else
				PostMessage( hwnd, WM_COMMAND, (IntPtr)command, IntPtr.Zero );
		}

		#endregion
	}

	sealed class Localizer : LocalizedStringProvider
	{
		private string parent;

		public Localizer()
		{
			this.parent = CultureInfo.CurrentCulture.Parent.Name;
		}

		public override string Author
		{
			get
			{
				return "Quizo";
			}
		}

		public override string Description
		{
			get
			{
				if( this.parent == "ja" )
				{
					return "GNXv[̃Xe[^Xo[̕\؂ւL[{[hV[gJbg񋟂܂B";
				}
				else
				{
					return "Provides shortcut key to show and hide status bar of Explorer.";
				}
			}
		}

		public override string Name
		{
			get
			{
				if( this.parent == "ja" )
				{
					return "Xe[^Xo[";
				}
				else
				{
					return "StatusBar Key";
				}
			}
		}

		public string ShortcutKeyString
		{
			get
			{
				if( this.parent == "ja" )
				{
					return "Xe[^Xo[̕\؂ւ";
				}
				else
				{
					return "Show/hide statusbar";
				}
			}
		}

		public override void SetKey( int iKey )
		{
		}
	}
}
